@@ -12,16 +12,8 @@
 #    License for the specific language governing permissions and limitations
 #    under the License.
 
-import struct
 import sys
 
-if struct.calcsize("P") == 8 and sys.platform == 'win32':
-    # This is needed by Nano Server.
-    # Set COINIT_MULTITHREADED only on x64 interpreters due to issues on x86.
-    import pythoncom
-    sys.coinit_flags = pythoncom.COINIT_MULTITHREADED
-    pythoncom.CoInitializeEx(pythoncom.COINIT_MULTITHREADED)
-
 from oslo_log import log as oslo_logging
 
 from cloudbaseinit import conf as cloudbaseinit_conf
