@@ -61,6 +61,8 @@ class TestWindowsConfigDriveManager(unittest.TestCase):
         self.osutils = mock.Mock()
         self._config_manager._osutils = self.osutils
         self.snatcher = testutils.LogSnatcher(module_path)
+        self._fake_label = 'config-2'
+        self._fake_metadata_file = 'fake_metadata_file'
 
     @mock.patch('os.path.exists')
     def _test_check_for_config_drive(self, mock_exists, exists=True,
@@ -70,7 +72,8 @@ class TestWindowsConfigDriveManager(unittest.TestCase):
         mock_exists.return_value = exists
 
         with self.snatcher:
-            response = self._config_manager._check_for_config_drive(drive)
+            response = self._config_manager._check_for_config_drive(
+                drive, self._fake_label, self._fake_metadata_file)
 
         self.osutils.get_volume_label.assert_called_once_with(drive)
         if exists and not fail:
@@ -256,11 +259,15 @@ class TestWindowsConfigDriveManager(unittest.TestCase):
             checks[2] = False
         mock_check_for_config_drive.side_effect = checks
 
-        response = self._config_manager._get_config_drive_from_cdrom_drive()
+        response = self._config_manager._get_config_drive_from_cdrom_drive(
+            self._fake_label, self._fake_metadata_file
+        )
 
         self.osutils.get_cdrom_drives.assert_called_once_with()
         idx = 3 if found else 4
-        check_calls = [mock.call(drive) for drive in drives[:idx]]
+        check_calls = [
+            mock.call(drive, self._fake_label,
+                      self._fake_metadata_file) for drive in drives[:idx]]
         mock_check_for_config_drive.assert_has_calls(check_calls)
         if found:
             mock_os_rmdir.assert_called_once_with(
@@ -287,7 +294,9 @@ class TestWindowsConfigDriveManager(unittest.TestCase):
         self.osutils.get_physical_disks.return_value = paths
         mock_extract_iso_from_devices.return_value = True
 
-        response = self._config_manager._get_config_drive_from_raw_hdd()
+        response = self._config_manager._get_config_drive_from_raw_hdd(
+            self._fake_label, self._fake_metadata_file)
+
         mock_map.assert_called_once_with(Disk, paths)
         self.osutils.get_physical_disks.assert_called_once_with()
         mock_extract_iso_from_devices.assert_called_once_with(
@@ -306,7 +315,8 @@ class TestWindowsConfigDriveManager(unittest.TestCase):
 
         with testutils.LogSnatcher('cloudbaseinit.metadata.services.'
                                    'osconfigdrive.windows') as snatcher:
-            response = self._config_manager._get_config_drive_from_vfat()
+            response = self._config_manager._get_config_drive_from_vfat(
+                self._fake_label, self._fake_metadata_file)
 
         self.assertTrue(response)
         self.osutils.get_physical_disks.assert_called_once_with()
@@ -340,7 +350,9 @@ class TestWindowsConfigDriveManager(unittest.TestCase):
         extract_calls = [mock.call(disk.partitions())
                          for disk in disks[:idx]]
 
-        response = self._config_manager._get_config_drive_from_partition()
+        response = self._config_manager._get_config_drive_from_partition(
+            self._fake_label, self._fake_metadata_file
+        )
         self.osutils.get_physical_disks.assert_called_once_with()
         mock_extract_iso_from_devices.assert_has_calls(extract_calls)
         self.assertEqual(found, response)
@@ -364,9 +376,13 @@ class TestWindowsConfigDriveManager(unittest.TestCase):
         checks = [False, found, found]
         mock_check_for_config_drive.side_effect = checks
         idx = 3 - int(found)
-        check_calls = [mock.call(volume) for volume in volumes[:idx]]
+        check_calls = [
+            mock.call(volume, self._fake_label,
+                      self._fake_metadata_file) for volume in volumes[:idx]]
 
-        response = self._config_manager._get_config_drive_from_volume()
+        response = self._config_manager._get_config_drive_from_volume(
+            self._fake_label, self._fake_metadata_file
+        )
         self.osutils.get_volumes.assert_called_once_with()
         mock_check_for_config_drive.assert_has_calls(check_calls)
         if found:
@@ -384,11 +400,13 @@ class TestWindowsConfigDriveManager(unittest.TestCase):
 
     def _test__get_config_drive_files(self, cd_type, cd_location,
                                       func, found=True):
-        response = self._config_manager._get_config_drive_files(cd_type,
-                                                                cd_location)
+        response = self._config_manager._get_config_drive_files(
+            self._fake_label, self._fake_metadata_file,
+            cd_type, cd_location)
         if found:
             if func:
-                func.assert_called_once_with()
+                func.assert_called_once_with(self._fake_label,
+                                             self._fake_metadata_file)
                 self.assertEqual(func.return_value, response)
         else:
             self.assertFalse(response)
@@ -450,18 +468,24 @@ class TestWindowsConfigDriveManager(unittest.TestCase):
                                      found=True):
         check_types = ["iso", "vfat"] if found else []
         check_locations = ["cdrom", "hdd", "partition"]
-        product = list(itertools.product(check_types, check_locations))
-        product_calls = [mock.call(cd_type, cd_location)
-                         for cd_type, cd_location in product]
+        product = list(itertools.product(check_types, check_locations,
+                                         [self._fake_label]))
+        product_calls = [mock.call(self._fake_label, self._fake_metadata_file,
+                                   cd_type, cd_location)
+                         for cd_type, cd_location, _ in product]
         mock_get_config_drive_files.side_effect = \
             [False] * (len(product_calls) - 1) + [True]
-        expected_log = ["Looking for Config Drive %(type)s in %(location)s" %
-                        {"type": cd_type, "location": cd_location}
-                        for cd_type, cd_location in product]
+        expected_log = [("Looking for Config Drive %(type)s in %(location)s"
+                         " with expected label %(label)s") %
+                        {"type": cd_type, "location": cd_location,
+                         "label": self._fake_label}
+                        for cd_type, cd_location, config_type in product]
 
         with self.snatcher:
             response = self._config_manager.get_config_drive_files(
+                self._fake_label, self._fake_metadata_file,
                 check_types, check_locations)
+
         mock_get_config_drive_files.assert_has_calls(product_calls)
         self.assertEqual(expected_log, self.snatcher.output)
         self.assertEqual(found, response)
